//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  crccalc.h (libcoyote)
//
//  Defines tools for calculating 32-bit cyclic-redundancy (CRC)
//  values from arrays of 8-bit byte data.
//---------------------------------------------------------------------
//
//  COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:
//
//  If you modify this file, you may insert additional notices
//  immediately following this sentence.
//  
//  Copyright 2000, 2002, 2003 Scott Robert Ladd.
//  All rights reserved, except as noted herein.
//
//  This computer program a_source file is supplied "AS IS". Scott Robert
//  Ladd (hereinafter referred to as "Author") disclaims all warranties,
//  expressed or implied, including, without limitation, the warranties
//  of merchantability and of fitness for any purpose. The Author
//  assumes no liability for direct, indirect, incidental, special,
//  exemplary, or consequential damages, which may result from the use
//  of this software, even if advised of the possibility of such damage.
//  
//  The Author hereby grants anyone permission to use, copy, modify, and
//  distribute this a_source code, or portions hereof, for any purpose,
//  without fee, subject to the following restrictions:
//  
//      1. The origin of this a_source code must not be misrepresented.
//  
//      2. Altered versions must be plainly marked as such and must not
//         be misrepresented as being the original a_source.
//  
//      3. This Copyright notice may not be removed or altered from any
//         a_source or altered a_source distribution.
//  
//  The Author specifically permits (without fee) and encourages the use
//  of this a_source code for entertainment, education, or decoration. If
//  you use this a_source code in a product, acknowledgment is not required
//  but would be appreciated.
//  
//  Acknowledgement:
//      This license is based on the wonderful simple license that
//      accompanies libpng.
//
//-----------------------------------------------------------------------
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//-----------------------------------------------------------------------

// Standard C++ library
#include <cstddef>

// libcoyote
#include "libcoyote/crccalc.h"
using namespace libcoyote;

// initialize a CRC table of precalculated values
crc_calculator::crc_precalc::crc_precalc()
{
    for (crc32_t n = 0; n < 256; ++n)
    {
        crc32_t c = n; // c will be the constant for n

        // assumes 8-bit bytes
        for (int bit = 0; bit < 8; ++bit)
        {
            if (c & 1)
                c = 0xEDB88320L ^ (c >> 1);
            else
                c >>= 1;
        }

        m_table[n] = c;
    }
}

// static member initialization
crc_calculator::crc_precalc crc_calculator::s_table;

// constructor
crc_calculator::crc_calculator()
{
    // initialize CRC value
    m_crc = 0xFFFFFFFFL;
}

// update CRC value based on an array of bytes
void crc_calculator::update(const unsigned char * data, size_t len)
{
    if (data == NULL)
        return;

    for (size_t n = 0; n < len; ++n)
        m_crc = s_table[(m_crc ^ data[n]) & 0xFF] ^ (m_crc >> 8);
}

