//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  maze_renderer.h (libcoyote)
//
//  Maze generation and exploration tools
//-----------------------------------------------------------------------
//
//  COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:
//  
//  If you modify this file, you may insert additional notices
//  immediately following this sentence.
//  
//  Copyright 2001, 2002, 2003 Scott Robert Ladd.
//  All rights reserved, except as noted herein.
//  
//  This computer program source file is supplied "AS IS". Scott Robert
//  Ladd (hereinafter referred to as "Author") disclaims all warranties,
//  expressed or implied, including, without limitation, the warranties
//  of merchantability and of fitness for any purpose. The Author
//  assumes no liability for direct, indirect, incidental, special,
//  exemplary, or consequential damages, which may result from the use
//  of this software, even if advised of the possibility of such damage.
//  
//  The Author hereby grants anyone permission to use, copy, modify, and
//  distribute this source code, or portions hereof, for any purpose,
//  without fee, subject to the following restrictions:
//  
//      1. The origin of this source code must not be misrepresented.
//  
//      2. Altered versions must be plainly marked as such and must not
//         be misrepresented as being the original source.
//  
//      3. This Copyright notice may not be removed or altered from any
//         source or altered source distribution.
//  
//  The Author specifically permits (without fee) and encourages the use
//  of this source code for entertainment, education, or decoration. If
//  you use this source code in a product, acknowledgment is not required
//  but would be appreciated.
//  
//  Acknowledgement:
//      This license is based on the wonderful simple license that
//      accompanies libpng.
//
//-----------------------------------------------------------------------
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//  
//-----------------------------------------------------------------------

#if !defined(LIBCOYOTE_MAZERENDERER_H)
#define LIBCOYOTE_MAZERENDERER_H

#include "libcoyote/maze.h"

namespace libcoyote
{
    //! Renders a <i>maze</i> object
    /*!
        A <i>maze_renderer</i> creates an image of a maze as a black-and-white
        PNG (Portable Network Graphics) file. This class requires libpng and zlib.
    */
    class maze_renderer
    {
    public:
        //! Renders a maze to a PNG file with a specific cell size.
        /*!
            Renders a maze object to a PNG file, using a specified cell size.
            \param a_source - The maze to be rendered
            \param a_filename - The name of the file to be created
            \param a_grid_size - The dimension (in pixels) of a rendered cell (always square)
        */
        static void render(const libcoyote::maze & a_source, const std::string & a_filename, size_t a_grid_size);

    private:
        // A type for handling mage images
        class image
        {
        public:
            // constructor
            image(const maze & source, size_t grid_size);

            // destructor
            ~image();

            // properties
            unsigned char ** get_bits() const
            {
                return m_bits;
            }

            int get_width() const
            {
                return m_width;
            }

            int get_height() const
            {
                return m_height;
            }

        private:
            // maze width
            int  m_width;

            // naze height
            int  m_height;

            // bits representing the maze
            unsigned char ** m_bits;
        };
    };
}

#endif
